/*  KRunning - CreateTables.cpp
 *  Copyright (C) 2003 by Andreas Scherf
 *  Begin Mon Apr 14 2003 write to scherfa@web.de if you found a bug.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "CreateTables.h"
#include <stdlib.h>
using namespace std;

CreateTables::CreateTables (QWidget * parent, const char *name) : CreateTablesGUI(parent,name,true) {
    connect (CancelButton,SIGNAL(clicked()),this,SLOT(slotQuit()));
    connect (OkButton,SIGNAL(clicked()),this,SLOT(slotOk()));
    connect (FileChooseButton,SIGNAL(clicked()),this,SLOT(slotChooseDatabaseFile()));
    connect (SelectButtonGroup,SIGNAL(clicked(int)),this,SLOT(slotSelectionChanged(int)));
    connect(DriverComboBox, SIGNAL(activated (const QString &)), SLOT(sqlDriverChanged(const QString &)));
    NewDbNameEdit->setText(i18n("NewDatabaseName"));
    NewDbNameEdit->setFocus();
    slotSelectionChanged(3);

    config = KGlobal::config();
    config->setGroup("RecentFiles");
    fileList=QStrList();
    config->readListEntry("Files",fileList);
    RecentFileCombo->insertStrList(fileList);

    config->setGroup( "Database" );

    DriverComboBox->insertStringList(dbconn->getDriverList());
    DriverComboBox->insertItem("SQLITE");
    DriverComboBox->setCurrentText(config->readEntry("Driver","SQLITE"));
    QToolTip::add(DriverComboBox,i18n("Select the database driver."));
    DriverTextLabel->setText(i18n("Sqlite the internal database."));

    UserNameEdit->setText(config->readEntry( "Username",getenv( "USER" ) ));

    LocationEdit->setText(config->readEntry( "Hostname","localhost" ));
    DatabaseNameEdit->setText( config->readEntry( "Databasename","lauf" ));
    PortInput->setValue(config->readEntry( "Port","5432" ).toInt());
    PasswordEdit->setText( QString(KCodecs::base64Decode(config->readEntry( "Password","" ).latin1())));

    QToolTip::add (DatabaseNameEdit, i18n ("Enter the name of the database."));
    QToolTip::add (PortInput, i18n ("Enter the port number. Default is 5432."));
    QToolTip::add (LocationEdit, i18n ("Enter the location (i.e. localhost, IP, etc)."));
    QToolTip::add (UserNameEdit,i18n	("Enter your database username (i.e. root, blank, etc)."));
    QToolTip::add (PasswordEdit, i18n ("Enter your password if you have one."));

    driverName=getDriver();
    connectTextMessage->setReadOnly(true);
}

CreateTables::~CreateTables() {
}


void CreateTables::slotQuit() {
    this->close();

}
void CreateTables::sqlDriverChanged(const QString &name) {
    driverName=name;
    if (driverName.compare("SQLITE")==0) {
        widgetStack->raiseWidget(0);
        DriverTextLabel->setText(i18n("Sqlite the internal database."));
    } else {
        widgetStack->raiseWidget(1);
        DriverTextLabel->setText(i18n(name +" an external database driver."));
    }

}

void CreateTables::slotOk() {
  QString dbname;
    if (driverName.compare("SQLITE")==0) {
        if (RadioEmpty->isChecked()) {
            filename=NewDbNameEdit->text();
            if (!filename.endsWith(".db") && !filename.endsWith(".DB")) {
                filename.append(".db");
                }
            dbname=QDir::currentDirPath()+'/'+filename;     
            dbconn->connect(dbname);
            if (fileList.find(dbname)==-1) {
             fileList.append(dbname);
             config->setGroup("RecentFiles");
             config->writeEntry("Files",fileList);
             config->sync();
            }          
        }
        else if (RadioRecent->isChecked()) {
            filename=RecentFileCombo->currentText();
            dbconn->connect(filename);
        }
        else if (RadioExisting->isChecked()) {
            dbconn->connect(filename);
        }
        if (!dbconn->testTables()) {
            dbconn->createEmptyTables();
        }
    } else {
        dbconn->setDefaultDB(  driverName );
        dbconn->connect(getName(),getUser(),getPasswd(),getLocation());
        if (dbconn->isConnected()) {
            // store new settings ...
            KConfig* kc=KGlobal::config();
            kc->setGroup( "Database" );
            kc->writeEntry( "Databasename",getName() );
            kc->writeEntry( "Username",getUser() );
            kc->writeEntry( "Password",  KCodecs::base64Encode(getPasswd().latin1()).data())  ;
            kc->writeEntry( "Location", getLocation() );
            kc->writeEntry( "Port", getPort() );
            kc->writeEntry( "Driver", getDriver() );
            kc->sync();
        }
        connectTextMessage->setColor(QColor(255,0,0));
        connectTextMessage->append("WARNING !!\nCouldn't connect to "+getName()+" Database.");
        return;
    }




    this->accept();
}

void CreateTables::slotSelectionChanged(int num) {
    switch (num) {
    case 0:
        RadioExisting->setChecked(true);
        FileChooseButton->setEnabled(true);
        RecentFileCombo->setEnabled(false);
        NewDbNameEdit->setEnabled(false);
        break;
    case 2:
        RadioRecent->setChecked(true);
        FileChooseButton->setEnabled(false);
        RecentFileCombo->setEnabled(true);
        NewDbNameEdit->setEnabled(false);
        break;
    case 3:
        RadioEmpty->setChecked(true);
        FileChooseButton->setEnabled(false);
        RecentFileCombo->setEnabled(false);
        NewDbNameEdit->setEnabled(true);
        break;
    }
}


void CreateTables::slotChooseDatabaseFile() {

    url=KFileDialog::getOpenURL(QDir::currentDirPath(),
                                i18n("*.db *.DB |SQLite files\n*|All files"), this, i18n("Load Database ..."));
    if (!url.isValid()) return;
    filename=url.directory()+'/'+url.fileName();
    DbNameLabel->setText(filename);
    if (fileList.find(filename)==-1) {
    fileList.append(filename);
    config->setGroup("RecentFiles");
    config->writeEntry("Files",fileList);
    config->sync();
    }
}

// Interfaces
QString CreateTables::getDriver () const {
    return DriverComboBox->currentText();
}

/** Returns the name */
QString CreateTables::getName () const {
    QString result;
    if ( DriverComboBox->currentText()=="SQLITE") {
        result=QString();
    } else {
        result= DatabaseNameEdit->text();
    }
    return result;
}

/** Returns the location */
QString CreateTables::getLocation () const {
    return   LocationEdit->text ();
}

/** Returns the password. */
QString  CreateTables::getPasswd () const {
    return PasswordEdit->text ();
}

/** Returns the port */
QString CreateTables::getPort () const {
    return QString::number(PortInput->value ());
}

/** Returns the username. */
QString CreateTables::getUser () const {
    return UserNameEdit->text ();
}
