/*  KRunning - DBConnection.h
 *  Copyright (C) 2002 by Andreas Scherf
 *  Begin Sam Mai 11 2002 write to scherfa@web.de if you found a bug.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
#ifndef DBCONNECTION_H
#define DBCONNECTION_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#define DIALOG_MARGIN 15
#include <qsqldatabase.h>
#include <qsqlcursor.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qvaluelist.h>
#include <qtooltip.h>
#include <qwidgetstack.h> // DBConnectioDialog
#include <qpushbutton.h>

// Settings & i18n
#include <kconfig.h>
#include <kglobal.h>
#include <klocale.h>
#include <kdialogbase.h>
#include <klineedit.h>
#include <kmdcodec.h> // Passwort decode DBConnection dialog -> better in Preferences ?
#include <kiconloader.h> // DBConnectioDialog
#include <kfiledialog.h>

// SQLite
#include "./sqlite/sqlite.h"
/** eventObj
@short Stores an event object
*/
class eventObj {
    public:
        enum UnitNum {
            Kilometers=0,
            Meters,
            Miles
    };

        eventObj(): number(-1) {}
        eventObj( int num,const QString& nm , const QString &snf,QTime stime,int dist )
                : number(num), name(nm), startnumberframes(snf),starttime(stime),distance(dist)
        {}
        QString getEventName() const { return name; }
        int getEventNumber() const { return number; }
        QTime getEventStarttime() const {return starttime;}
        int getEventDistance() const {return distance;}
        QString getStartnumberFrames() const {return startnumberframes;}

        QString eventstarttime (const QString &format) const {
            return starttime.toString(format);
        }

        void addStartNumberFrame(const QString &frame) {
            startnumberframes.append(frame+';');
        }
        void setStartNumberFrame(const QString &frame) { startnumberframes=frame; }
        void setDistance(int dist) {distance=dist;}
        void setEventName(const QString &nm){name=nm;}
        void setStartTime(const QTime &time){starttime=time;}

        bool save (bool update);
        bool save();
    private:
        int number;
        QString name;
        QString startnumberframes;
        QTime starttime;
        int distance;
        UnitNum unit;
};
static const QString unitString[]={I18N_NOOP("km"),
                                   I18N_NOOP("m"),
                                   I18N_NOOP("miles")};
/**
@short Stores a team object
*/

class teamObj {
    public:
        teamObj(): number(-1){}
        teamObj(int num,const QString &nm)
                :number(num),name(nm) {}
        bool save (bool update);
        bool save();
        QString teamname() const {return name;}
        int teamnumber() const {return number;}
    private:
        int number;
        QString name;
};
//number int primary key, surname text,forename text,birthday int2,sex int2,
//event int references events,team int references teams,street text,city text,
//plz int, chipnum text, state int2,place_total int2,place_class int2, place_sex int2, place_team int2,place_nation int2)");
class runnerObj {
    public:
        enum SexNum {
            Male=0,
            Female,
            Unknown
    };
        enum StateNum {
            Created=0,
            Evaluated,
            Ready
    };
        runnerObj(): number(-1){}
        runnerObj(int num,const QString& sn,const QString& fn,int bthd,SexNum sx,int ev,int tm,const QString& st,const QString& ct,int p,const QString& nt,
                  const QString& cn,StateNum sa,const QString& com,int placetotal,int placeclass,int placesex,int placeteam,int placenation) : number(num),surname(sn),forename(fn),birthday(bthd),sex(sx),event(ev),team(tm),
                street(st),city(ct),plz(p),nation(nt),chipnum(cn),state(sa),comment(com),placetotal(placetotal),placeclass(placeclass),placesex(placesex),placeteam(placeteam),placenation(placenation),newnumber(num)
        {}

        runnerObj(int num,const QString& sn,const QString& fn,int bthd,SexNum sx,int ev,const QString& tn,const QString& st,const QString& ct,int p,const QString& nt,
                  const QString& cn,StateNum sa,const QString& com,int placetotal,int placeclass,int placesex,int placeteam,int placenation) : number(num),surname(sn),forename(fn),birthday(bthd),sex(sx),event(ev),teamname(tn),
                street(st),city(ct),plz(p),nation(nt),chipnum(cn),state(sa),comment(com),placetotal(placetotal),placeclass(placeclass),placesex(placesex),placeteam(placeteam),placenation(placenation),newnumber(num)
        {}

        int runnernumber() const { return number; }
        void setNewRunnerNumber(int num) {newnumber=num;}
        SexNum runnersex() const { return sex; }
        int runnerbirthday() const { return birthday; }
        int runnerevent() const {return event;}
        int runnerteam() const {return team;}
        int runnerplz() const {return plz;}
        QString runnercity() const {return city;}
        QString runnerstreet() const {return street;}
        QString runnernation() const {return nation;}
        StateNum runnerstate() const {return state;}
        QString runnerteamname() const {return teamname;}
        QString runnersurname() const {return surname;}
        QString runnerforename() const {return forename;}
        QString runnercomment() const {return comment;}
        int getTotalPlace() const {return placetotal;}
        int getClassPlace() const {return placeclass;}
        int getSexPlace() const {return placesex;}
        int getTeamPlace() const {return placeteam;}
        int getNationPlace() const {return placenation;}
        // For evaluation ...
        void setState (StateNum s) {state=s;}
        void setTotalPlace(int t) {placetotal=t;}
        void setClassPlace(int c) {placeclass=c;}
        void setSexPlace(int s) {placesex=s;}
        void setTeamPlace(int t) {placeteam=t;}
        void setNationPlace(int n) {placenation=n; }
        bool save (bool update);
        bool save ();

    private:
        int number;
        QString surname;
        QString forename;
        int birthday;
        SexNum sex;
        int event;
        int team;
        QString teamname;
        QString street;
        QString city;
        int plz;
        QString nation;
        QString chipnum;
        StateNum state;
        QString comment;
        int placetotal;
        int placeclass;
        int placesex;
        int placeteam;
        int placenation;
        int newnumber; // For changing the number ...
};
// This is for qBubbleSort and qHeapSort functions in class evaluation
extern bool operator<(runnerObj &,runnerObj &);

class timeObj {
    public:
        timeObj(): number(-1){}
        timeObj(int n,int e,QTime t):number(n),event(e),time(t){}
        timeObj(const QTime& t);
        bool save (bool update);
        bool save();
        int timenumber() const {return number;}
        int eventnumber() const {return event;}
        QTime getTime() const {return time;}
        QString getTimeString(const QString & ts);
    private:
        int number;
        int event;
        QTime time; // its a lot better to store a qtime objekt here ...
};
// This is for qBubbleSort and qHeapSort functions
extern bool operator<(timeObj &,timeObj &);

/** This class stores the relation between times and runners ...
*/
class runnertimeObj {
    public:
        runnertimeObj():number(-1){}
        runnertimeObj(int n,int e,int r,int t):number(n),event(e),runner(r),time(t){}
        bool save();
        int timenumber() const {return time;}
        int runnernumber() const{return runner;}
    private:
        int number;
        int event;
        int runner;
        int time;
} ;
/**
@short Main Document class holds the Document Name
*/
class documentObj {
    public:
        documentObj():number(-1){}
        documentObj(int n,const QString & na,const QString & si):number(n),name(na),size(si){}
        bool save (bool update);
        bool save();
        QString documentname() const {return name;}
        QString documentsize() const  {return size;}
    private:
        int number;
        QString name;
        QString size;
};
/**
@short This class stores the relation between documents and the frames inside the document
*/
class documentframeObj {
    public:
        documentframeObj():number(-1){}
        documentframeObj(int n,int d,int f):number(n),document(d),frame(f){}
        bool save();
        int documentnumber() const {return document;}
        int framenumber() const {return frame;}
    private:
        int number;
        int document;
        int frame;
};

/**
@short The frame class holds the frame attributes such as text,font, position ...
*/
class frameObj {
    public:
        frameObj():number(-1){}
        bool save (bool update);
        bool save();
    private:
        int number;
};

/** Not used yet.
*/
class statisticObj {
    public:
        statisticObj();
};
/** Main Class wich interacts between the KRunning Obj and a database (Postgres).
@author Andreas Scherf
@version 0.3
@short Layer between the database and the Objects
*/
class DBConnection {
        // Q_OBJECT
    public:
        enum DBType {
            SQLite=0,
            Postgres,
            MySQL,
            Unknown
    };
        DBConnection();
        virtual DBConnection::~DBConnection();
        void setDefaultDB(const QString &driver);
        // void setDBParameters();
        void connect ();
        void connect(const QString &dbname,const QString &username,const QString &dbpasswd,const QString &dbhost);
        void connect(const QString &);
        void connectExternalDb();
        void connectSqlite (const QString &);
        void disconnect();

        bool isConnected();
        bool testTables();
        bool createEmptyTables();
        QSqlDatabase *getDatabase();
        // SQLite
        DBType getDBType() const {return dbtype;}
        sqlite * getSQLiteDB() const {return db;}
        void setCurrentEvent(const QString &name);
        void setCurrentEvent(int num);

        eventObj getCurrentEvent();

        void startTransaction();
        void commitTransaction();

        QStringList getDriverList();
        QString getDBVersion();
        QString getCountString(const QString & );
        QString getDatabaseName();
        void setDatabaseName (QString name){dbname=name;}
        KConfig* getConfig(){return krunningConfig;}

    public:

        typedef QValueList<eventObj> EventList;
        typedef QValueList<teamObj> TeamList;
        typedef QValueList<timeObj> TimeList;
        typedef QValueList<runnerObj> RunnerList;
        typedef QValueList<runnertimeObj> RunnerTimeList;


        // String Lists for Combo Boxes ...
        QStringList getTeamNameList();
        QStringList getEventNameList();
        // And the AutopCompletion
        QStringList getNameList(bool name);
        // Get all Startnumberframes
        QStringList getStartNumberFrames();

        // Lists
        RunnerList getRunnerListByEvent (int event,bool join);
        RunnerList getRunnerListByEventAndSex (int event,runnerObj::SexNum sex);
        RunnerTimeList getRunnerTimeListByEvent (int event,bool join);
        TimeList getTimeListByEvent(int event);
        EventList getEventList();
        TeamList getTeamList();
        // Obj
        eventObj getEventByStartnumber(int number);
        eventObj getEventByNumber (int number);
        eventObj getEventByName(const QString &name);
        timeObj getTimeByNumber(int number);
        runnerObj getRunnerByNumber (int number);
        QString getEventNameByNumber (int number);
        QString getTeamNameByNumber (int number);
        int getTeamnumberByName (const QString &team);
        int getEventnumberByName (const QString &event);
        void query(QString &q);
        // Delete
        bool deleteTimeByTimestring(const QString & time,int event);
        bool deleteTeamByString( const QString & team);
        // Checks
        bool checkIfRunnerInEvent(int runner,int event);
        bool checkIfRunnerHasTime(int  runner,int event);
        int DBConnection::testStartNumberFrame(QString & frame);
        // SQLite
        QStringList getSqliteTables();
    private:
        QStringList driverList;
        QSqlDatabase *defaultDB;
        KConfig* krunningConfig; // Obsolte !!
        bool connected;
        eventObj currentEvent;
        // SQLite
        DBType dbtype;
        sqlite *db;
        QString dbname;
};


extern DBConnection *dbconn;

#endif // DBCONNECTION_H
