/*  KRunning - DigitalClock.h
 *  Copyright (C) 2002 by Andreas Scherf
 *  Begin Mon Mai 6 2002 write to scherfa@web.de if you found a bug.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef DIGITALCLOCK_H
#define DIGITALCLOCK_H

#include <qlcdnumber.h>
#include <qdatetime.h>
#include <qframe.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qgroupbox.h>
#include <qtimer.h>

class DigitalClock : public QWidget
{
        Q_OBJECT
    public:
        DigitalClock (QWidget * parent = 0, const char *name = 0);
        virtual         DigitalClock::~DigitalClock ();
        void            setStartTime (const QString & time);
        void            setMessageText (const QString & text);
        void            setMessageText (const int &number);
        void            setMessageFont (const QFont & font);
        void            setLabelText (const QString & text);
        QString         getCurrentRunningTime ();
        void            setShowCTime(bool flag);
        QTime           getTimeDiff (const QTime & t);
        void            setDisplayTime(const QString &dtime);
        void            start ();
        void setTimeFormat(const QString &ts);
    protected:			// event handlers
        void            timerEvent (QTimerEvent *);
        void            mousePressEvent (QMouseEvent *);

    private slots:	// internal slots
        void            stopStartTime ();
        void            showTime ();

    private:			// internal data
        QLCDNumber * lcdNumber;
        QGroupBox      *mainFrame;
        QFrame         *sepLine;
        QVBoxLayout    *vBox;
        QLabel         *label;
        QLabel         *message;

        void            showStartTime ();
        bool            showingColon;
        int             normalTimer;
        int             showStartTimer;
        bool            showStartTimeOnly;
        bool            showCurrentTime;
        int             runnumber;
        //bool showMSecs;
        QString timestring;
        QTime           starttime;
};

#endif // COUNTTIME_H

