/*  KRunning - Time.cpp
 *  Copyright (C) 2002 by Andreas Scherf
 *  Begin Mon Mai 6 2002 write to scherfa@web.de if you found a bug.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "Time.h"

Time::Time (QWidget * parent, const char *name) : TimeGUI (parent, name, true)
{
    AnalogClock1->setLabelText( i18n("Current time"));
    AnalogClock2->setLabelText( i18n("Finished Runners"));
    AnalogClock3->setLabelText( i18n("Actual Running Time"));
    AnalogClock3->setTimeFormat(prefs->getTimeFormat()); //Configure ??
    AnalogClock1->setShowCTime(TRUE);
    AnalogClock1->start();
    AnalogClock3->start();
    AnalogClock2->setDisplayTime("00:00:00");
    AnalogClock2->setMessageText(0);
    // Set the Message font size to 20 ...
    QFont font=KApplication::font();
    font.setPointSize(20);
    AnalogClock2->setMessageFont(font);

    TimeListView->clear();

    LineEdit1->setReadOnly ( TRUE );
    NumberInput->setValidChars("0123456789"); // Only Numbers could be entered ..
    if (dbconn->isConnected()) {
        filename=dbconn->getDatabaseName();
    } else {
        filename = QString ("");
        timesum = 1;
    }
    TimeListView->setResizeMode(QListView::AllColumns);
    TimeListView->addColumn      ( i18n( "Time" ));
    TimeListView->addColumn     ( i18n( "Number" ));
    popupMenu = new QPopupMenu(TimeListView,"popupMenu");
    popupMenu->setCaption("Time Menu");
    popupMenu->insertItem( i18n("delete Time"), this, SLOT(deleteSelectedTime()) );

    fullScreenMenu = new QPopupMenu(this,"fullScreenMenu");
    fullScreenMenu->insertItem( i18n("Show Full Screen"), this, SLOT(slotShowFullScreen()) );

    connect (TimeListView,SIGNAL(rightButtonPressed ( QListViewItem *, const QPoint &,int ) ),this,SLOT(showPopupMenu(QListViewItem*,const QPoint &,int)));
    //  connect(this,SIGNAL(mousePressEvent()),this,SLOT(mousePressEvent()));
    fullTime=0;
}

Time::~Time ()
{
}

//
// Stores aktual time to file ... ?? Im Time Code ???
//
void Time::keyPressEvent (QKeyEvent * k) {
    QTime time;
    int timenumber=-1,runnernumber=-1;
    bool runnernumbervalid=FALSE;
    if (k->key () == Qt::Key_Return || k->key() == Qt::Key_Enter) {
        time=AnalogClock3->getTimeDiff (QTime::currentTime ());
        if (dbconn->isConnected()) {
            if (NumberInput->text().length()>0)
                runnernumber=NumberInput->text().toInt();

            timeObj *timeObject=new timeObj(-1,dbconn->getCurrentEvent().getEventNumber(),time);
            timeObject->save();
            timenumber=timeObject->timenumber();
            delete (timeObject);
            if (dbconn->checkIfRunnerInEvent(runnernumber,dbconn->getCurrentEvent().getEventNumber())) { // Check here if the runner exists and if the runner is in this event
                // CheckIfRunnerhas a time ??
                if (dbconn->checkIfRunnerHasTime(runnernumber,dbconn->getCurrentEvent().getEventNumber())) {
                    runnernumbervalid=FALSE;
                } else {
                    runnertimeObj *runnertime = new runnertimeObj(-1,dbconn->getCurrentEvent().getEventNumber(),runnernumber,timenumber);
                    runnertime->save();
                    delete (runnertime);
                    runnernumbervalid=TRUE;
                }
            } else {
                runnernumbervalid=FALSE;
            }
        }/* else {
            QDir::setCurrent (QDir::homeDirPath ());
            fstream         FS (filename, ios::in | ios::out);
            FS.seekp (0, ios::end);
            FS << timenumber << " " << time.toString("hh:mm:ss.zzz") << endl;
            FS.close ();
        }  */

        // Do some output
        if (runnernumbervalid)   {
            TimeListView->insertItem(new TimeViewItem(TimeListView,time,NumberInput->text().toInt()));
        } else {
            TimeListView->insertItem(new TimeViewItem(TimeListView,time));   // Configure ??
        }

        timesum++; // Starts with 0 !!
        AnalogClock2->setDisplayTime(time.toString());
        AnalogClock2->setMessageText (timesum);
        TimeListView->ensureItemVisible ( TimeListView->lastItem()) ; // SO we could see always the last time
        NumberInput->clear();
    }
}


void Time::setTimeList() {
    DBConnection::TimeList list;
    DBConnection::RunnerTimeList rtlist;
    int runnernumber=-1;
    TimeListView->clear();
    timesum=0;
    list=dbconn->getTimeListByEvent(dbconn->getCurrentEvent().getEventNumber());
    rtlist=dbconn->getRunnerTimeListByEvent(dbconn->getCurrentEvent().getEventNumber(),false);

    for (  DBConnection::TimeList::Iterator it = list.begin(); it != list.end(); ++it ) {
        timeObj *time=new timeObj(*it);
        for (DBConnection::RunnerTimeList::Iterator itr=rtlist.begin();itr!=rtlist.end();itr++) {    // Searching for the runnernumber in the runnertimes relation
            runnertimeObj *runnertime=new runnertimeObj(*itr);
            if (time->timenumber()==runnertime->timenumber()) {
                runnernumber=runnertime->runnernumber();
            }
            delete (runnertime);
        }
        if (runnernumber==-1) {
            TimeListView->insertItem(new TimeViewItem(TimeListView,time->getTime()));   // Configure ??
        }else {
            TimeListView->insertItem(new TimeViewItem(TimeListView,time->getTime(),runnernumber));
        }
        runnernumber=-1;
        timesum++;
        AnalogClock2->setDisplayTime(time->getTimeString("hh:mm:ss"));
        delete (time);
    }
    AnalogClock2->setMessageText (timesum);
    TimeListView->ensureItemVisible ( TimeListView->lastItem()) ;
}

void Time::setDefaultSettings() {
    this->setStartTime(dbconn->getCurrentEvent().getEventStarttime());
    if (dbconn->isConnected()) {
        this->setFilename(dbconn->getDatabaseName());
    } else {
        this->setFilename(prefs->getFilename());
    }
    setTimeList();
    NumberInput->setFocus();
}

void Time::showPopupMenu (QListViewItem* item,const QPoint & point,int col) {
    popupMenu->exec(point);
}

void Time::mousePressEvent (QMouseEvent * e) {
    if (e->button () == QMouseEvent::RightButton)	{// right button pressed
        //        QPoint point =
        fullScreenMenu->exec(e->globalPos());
    }
}

void Time::slotShowFullScreen() {
    if (parentWidget()!=0) {
        if (fullTime==0)
            fullTime = new Time(0,"TimeWidget");

        fullTime->showFullScreen();
        fullTime->setFocusPolicy( QWidget::StrongFocus );
        fullTime->setDefaultSettings();
        fullTime->show();
    } else {
        this->close();
    }
}

void Time::deleteSelectedTime() {

    if ( dbconn->deleteTimeByTimestring( TimeListView->currentItem()->text(0),dbconn->getCurrentEvent().getEventNumber() )) {
        TimeListView->takeItem( TimeListView->currentItem());
        timesum--;
        AnalogClock2->setMessageText (timesum);
        AnalogClock2->setDisplayTime( TimeListView->lastItem()->text(0).left(8));
    }
}


/** getRTLCode - Decodes the Time that it matches the Timecode from RTL
 ** below there is the decode Sequence from KRunning ...
 ** seconds = (atoi (string[1]) / 100) % 100;                          // Sekunden
 ** seconds = seconds + (((atoi (string[1]) / 10000) % 60) * 60);      // Minuten
 ** seconds = seconds + (((atoi (string[1]) / 10000) / 60) * 3600);    // Stunden
 ** @author Andreas Scherf
 ** @version 1.0
 */
uint Time::getRTLCode (const QTime & t) {
    uint            result;
    result = t.second () * 100;
    result += t.minute () * 10000;
    result += (t.hour () * 600000);	///3600;  //ERROR
    return result;
}
uint Time::getSeconds (const QTime &t) {
    uint result;
    result = (t.hour()*3600);
    result += t.minute() *60;
    result += t.second();
    return result;
}

void Time::setFilename(const QString &name) {
    filename=strdup(name);
    LineEdit1->setText(name);
}

void Time::setStartTime(const QTime &time) {
    AnalogClock3->setStartTime(time.toString());
}


//----------------------
TimeViewItem::TimeViewItem (KListView *parent,const QTime & time,int number)
        :KListViewItem(parent)
{
    setText(0, time.toString("hh:mm:ss.zzz" ));
    if (number > 0)
        setText(1, QString::number(number) );
    else
        setText(1, "---");
}

int TimeViewItem::compare( QListViewItem * i, int col, bool ascending ) const
{
    TimeViewItem *other = (TimeViewItem *)i;
    switch ( col ) {
    case 1:
        if ( text(0).toInt() == other->text(0).toInt() )        // --
            return 0;
        else
            return text(0).toInt() < other->text(0).toInt() ? -1 : 1;
        break;
    default:
        // use default method for colum 0 and others added in the future
        return QListViewItem::compare( i, col, ascending );
    };
}
//------------------------------------------
